%% This is the main regenerator model developed at UW.  This function takes
%% inputs that define the system and forcing functions, such as mass flow
%% profile, and calculates the regenerator performance.  This model does
%% not account for heat exchangers.  The main outputs of the model are the
%% cyclical steady state temperatures of the fluid and solid over an entire
%% cycle and the heat rejection, cooling power, and pumping power of the
%% system.  This model is described in Progress Report #1.
%%
%% This model is based on version 1_2_4, which was developed at Astronautics
%% over the summer of 2006.  It was brought back to UW on August 28, 2006.
%% 3_2 changes the indexing slighttly so that the number of rows in the
%% temperature matrices as the time array.
%% 3_3 uses the heat transfer degradation factor from the Journal of Heat
%% Transfer paper and fixed bug in calculation of t_node
%% 3_4 the inputs to this function are modified to allow more flexibility
%% in the text input file.  The call to the magnetic field function has
%% been modified to be more flexible.
%% 3_5 accepts the mass flow rate and magnetic field as inputs to the
%% function.  These values are no longer calculated inside the function as
%% previous functions did.
%% 3_6 added additional outputs from the function, removed some unused
%% lines of code and other housekeeping.
%% 3_7 calls updated versions of pack_geom and bedcorrfunction.  The way
%% the regenerator material is specified has been corrected.
%% 3_8 changes the velocity array 'v' that is passed to the bedcorrfun.
%% Added function outputs.
%% 3_10 fixes a bug that caused the viscous dissipation to be calculated
%% incorrectly.  The problem was related to the calculation of dP.
%% 4_1 Include entropy generation term in the energy balance equations
%% related to the irreversible magnetization function to deal with magnetic hysteresis.
%% 4_2 Cleaned up code, modified some outputs, bypass MCMPROPS function to
%% calculate magnetocaloric properties of layered beds of a single material faster.
%% Bed paramters

tic;                % start timer to determine clock time needed to achieve convergence
dby=0;              % m, bypass flow pipe diameter
[dh,eps,as]=pack_geom(pacRAW,bedcorrfun);

extrapnum=3;        % number of cycles between extrapolations
extlin=6;           % linear number of cycles to extrapolate
order=1;            % polynomial fit order
wt=1;               % weighting function to extrapolations

i=1:Nx;
xf=L*(i'-1/2)/Nx;   % fluid spatial grid
xr=xf;              % regenerator spatial grid

dt=tau/(Nt-1);

imp=1;              % 1=run implicit to generate guess values, 0=don't run implicit

if imp==0
    
    %% start model with a ramp temperature profile
    pacCOOKED=[pacRAW,as,eps,dh,cNu,cff,cnk,L];  %inputs to the bed correlation function
    Tr(:,1)=TH-xr*(TH-TC)/L; %initial regenerator temp
    Tf(:,1)=TH-xf*(TH-TC)/L; %initial fluid temp
else
    
    %% call implicit UW model to generate initial guess values and speed
    %% model convergence
    
    pacCOOKED=[pacRAW,as,eps,dh,cNu,cff,cnk,L];
    [Trg,Tfg,tf,tr]=AMR_impguessT_6(impmcmpropfun,fluidfun,bedcorrfun,pacCOOKED,L,tau,Ac,Nx,mdot_amp,t_node,TH,TC,TCurie,nbed,mdot,muoH);
    Tr(:,1)=Trg(:,1);
    Tf(:,1)=Tfg(:,1);
end

%% Calculate energy difference between fluid at TH and TC.  These properties
%% are also used to calculated heat flux at the hot and cold reservoirs.
[mufC,kfC,cfC,rhofC,hfC,sfC]=fluidfun(TC);
[mufH,kfH,cfH,rhofH,hfH,sfH]=fluidfun(TH);
deltaUf=hfH-hfC;

%% initialize variables before beginning the main loop.  These variables are
%% used to check for convergence
done=0;
j=0;
ref_load=[-500 -500];
Er=[0 0];       %this matrix stores the total energy in the solid material for the past two cycles
Ef=[0 0];       %this matrix stores the total energy in the fluid for the past two cycles
Etot=[0 0];     %this matrix stores the total energy in the fluid and solid for the past two cycles
count=0;
deltaE=zeros(extrapnum,1);
err_s=1;

rhof=1000;
rhor=7170;

A=180; %for Ergun model from Fand et al
B=1.8; %for Ergun model from Fand et al
v=-mdot/(Ac*rhof);

T_ref = 273.3; %K
T_curie_ref=290;  %K,Not real curie temp, this is temperature of greatest entropy change.  T_c is magnetic field dependent.
a=TCurie/T_curie_ref;

f_muf=[-0.60568331 3.75601092 -9.32406002 11.58832650 -7.21611665 1.80330228];
f_cf = [-1288.18 5491.12 -7522.49 7526.83];
int_cp=polyint(f_cf);

h_0=[0,79.0701695647497,202.761777320551,329.741279456052,457.745164540650,586.704472008149,718.445630329500,851.380959214601,983.903028805001,1118.48324799610,1253.37425100530,1396.67921384750,1541.99898017990,1691.77093398605,1841.34088772885,1991.84880004435,2143.13932490455,2298.69018677050,2454.46030139440,2608.25927667400,2768.71559888300,2930.73923639300,3097.35734327690,3268.70216054290,3438.13816624870,3607.85309164870,3781.80858208195,3961.13850823445,4143.16630426405,4326.41156796985,4505.89092324970,4691.79144005870,4879.26081241820,5072.07753583015,5263.22515338895,5460.06982546855,5657.48770498945,5856.26351714295,6061.64489709315,6263.18409983055,6465.56060291645,6679.80343298445,6885.87459258025,7095.34649590025,7312.82041684225,7531.75728393350,7750.39035039990,7970.84742309390,8198.76282872140,8426.61887701940,8656.46038705300,8887.04787448240,9121.32476962120,9353.95422329520,9597.95162639710,9836.59731046310,10073.8848285808,10319.2013770207,10568.0806739100,10818.9556866917,11069.3579663737,11324.7577459697,11586.9977003169,11846.6248203549,12109.0050607562,12378.8158479053,12638.9076306983,12903.5417429551,13172.2137365942,13445.0322892686,13722.0430153926,13998.2337480984,14277.8231123760,14561.7140359692,14840.8796605499,15132.4924172852,15422.3389867460,15715.8668188310,16010.1201913233,16308.6841604268,16606.0526088009,16908.0884305879,17211.9115402234,17517.5692264206,17834.1946855796,18144.7922597834,18452.5136070608,18771.6299677335,19097.7067523035,19423.9515910555,19742.4010709749,20074.5450361293,20403.3079304997,20727.2566909677,21057.7744094493,21399.6721398452,21746.6773346276,22089.2337586941,22430.0788842297,22771.5681033722,23126.5237963565,23477.7684999312,23831.2127726545,24192.8635755345,24555.4045027625,24923.8900949546,25289.5541815886,25669.7368398689,26053.2490999049,26431.8929425316,26802.9718209137,27179.2468648937,27571.9838307982,27956.4107064260,28345.7924821854,28741.7516821855,29149.8642729844,29560.3993529723,29952.8071235458,30350.9118289858,30772.1861527233,31177.9468973307,31591.5967080999,32018.1490903812,32435.7414041411,32832.7409982713,33272.8137427014,33700.6479033271,34129.6940268612,34567.0361958095,35003.4078272175,35443.7100586327,35880.2007783354,36324.9812545027,36770.1999442225,37224.0585954039,37679.0665706782,38137.3927051276,38602.1962766928,39061.6179646362,39531.6392923392,40000.3690872897,40470.5033673503,40949.1369484872,41422.1235187741,41912.8808844591,42400.8816944671,42901.7533243928,43389.1069369042,43865.3508924322,44356.8741319591,44859.7901078415,45375.8177390245,45887.0550269845,46400.0818856080,46915.0709370864,47436.6678860882,47953.9056562572,48477.4242503656,49004.2245569971,49535.1084313668,50079.2918551865,50612.9958763827,51162.6908413492,51710.0888485414,52253.2712357612,52805.7852907748,53356.5695994118,53917.8716388698,54481.2675694043,55056.9663718964,55620.1779684099,56185.7629922049,56771.0763679829,57354.5051359739,57939.5553222659,58533.2427258284,59148.7252224526,59751.6537904426,60339.5620928269,60946.7110499154,61560.2705123930,62184.3125294811,62809.1161237531,63461.4732191860,64110.9817859614,64743.7301440439,65410.0743946541,66073.5102755976,66718.2001111499,67384.1003274619,68060.2435709036,68743.9211635368,69434.2962066621,70127.5373857038,70827.8464605693,71545.0436664984,72289.6165263424,73031.7181613085,73755.5824577462,74506.4392073892,75280.0325260691,76098.4414045836,76919.8368520434,77740.7243386014,78602.5303315150,79511.2587654025,80486.1706787303,81550.3771229199,83119.8403332031,85469.4031649381,87700.8705616203,88851.1904862008,89661.1886820232,90427.6146249407,91154.3073975881,91880.0143570007,92613.4913644464,93337.1510029852,94058.5176435667,94772.7473523580,95488.6681635477,96201.1986474300,96906.1883603125,97616.6688934483,98330.4195942076,99035.1358006971,99747.4633788207,100452.116579095,101160.928101704,101870.051150981,102577.852315388,103307.981487118,104026.507617024,104748.008730998,105469.934354060,106181.255104935,106912.600677299,107637.605968925,108364.453713105,109093.161947058,109835.892208766,110582.116463822,111328.913844921,112073.128794802,112815.478930900,113557.836378211,114303.037690324,115039.417181197,115792.290456453,116552.113520560,117315.669253914;]; %J/kg
T_0=[78.0470000000000;78.5020000000000;79.2000000000000;79.9060000000000;80.6120000000000;81.3170000000000;82.0260000000000;82.7330000000000;83.4290000000000;84.1270000000000;84.8190000000000;85.5430000000000;86.2690000000000;87.0100000000000;87.7420000000000;88.4720000000000;89.1980000000000;89.9370000000000;90.6700000000000;91.3860000000000;92.1260000000000;92.8660000000000;93.6190000000000;94.3850000000000;95.1360000000000;95.8820000000000;96.6390000000000;97.4120000000000;98.1880000000000;98.9620000000000;99.7150000000000;100.487000000000;101.257000000000;102.044000000000;102.818000000000;103.607000000000;104.393000000000;105.178000000000;105.982000000000;106.765000000000;107.546000000000;108.367000000000;109.149000000000;109.939000000000;110.754000000000;111.569000000000;112.377000000000;113.185000000000;114.015000000000;114.838000000000;115.662000000000;116.484000000000;117.315000000000;118.135000000000;118.989000000000;119.818000000000;120.637000000000;121.479000000000;122.326000000000;123.174000000000;124.018000000000;124.874000000000;125.746000000000;126.604000000000;127.466000000000;128.349000000000;129.195000000000;130.051000000000;130.917000000000;131.789000000000;132.669000000000;133.545000000000;134.429000000000;135.320000000000;136.189000000000;137.092000000000;137.985000000000;138.885000000000;139.783000000000;140.689000000000;141.587000000000;142.495000000000;143.405000000000;144.316000000000;145.256000000000;146.174000000000;147.077000000000;148.011000000000;148.961000000000;149.906000000000;150.824000000000;151.776000000000;152.714000000000;153.634000000000;154.570000000000;155.533000000000;156.505000000000;157.464000000000;158.416000000000;159.362000000000;160.339000000000;161.303000000000;162.268000000000;163.250000000000;164.231000000000;165.225000000000;166.207000000000;167.222000000000;168.242000000000;169.248000000000;170.230000000000;171.220000000000;172.250000000000;173.254000000000;174.271000000000;175.300000000000;176.353000000000;177.412000000000;178.419000000000;179.435000000000;180.505000000000;181.534000000000;182.581000000000;183.656000000000;184.705000000000;185.698000000000;186.799000000000;187.864000000000;188.924000000000;190.002000000000;191.074000000000;192.152000000000;193.218000000000;194.301000000000;195.385000000000;196.487000000000;197.586000000000;198.692000000000;199.804000000000;200.896000000000;202.012000000000;203.122000000000;204.235000000000;205.367000000000;206.479000000000;207.624000000000;208.759000000000;209.924000000000;211.055000000000;212.159000000000;213.296000000000;214.453000000000;215.633000000000;216.801000000000;217.971000000000;219.139000000000;220.318000000000;221.483000000000;222.660000000000;223.839000000000;225.025000000000;226.238000000000;227.423000000000;228.639000000000;229.842000000000;231.033000000000;232.244000000000;233.446000000000;234.666000000000;235.884000000000;237.123000000000;238.333000000000;239.543000000000;240.783000000000;242.013000000000;243.248000000000;244.498000000000;245.790000000000;247.050000000000;248.269000000000;249.520000000000;250.784000000000;252.061000000000;253.326000000000;254.643000000000;255.951000000000;257.216000000000;258.540000000000;259.853000000000;261.122000000000;262.424000000000;263.732000000000;265.047000000000;266.369000000000;267.680000000000;268.990000000000;270.323000000000;271.690000000000;273.033000000000;274.331000000000;275.661000000000;277.002000000000;278.391000000000;279.753000000000;281.077000000000;282.425000000000;283.775000000000;285.123000000000;286.420000000000;287.493000000000;288.788000000000;290.753000000000;292.586000000000;294.018000000000;295.419000000000;296.782000000000;298.171000000000;299.595000000000;301.012000000000;302.434000000000;303.851000000000;305.274000000000;306.696000000000;308.111000000000;309.539000000000;310.980000000000;312.413000000000;313.857000000000;315.286000000000;316.735000000000;318.187000000000;319.634000000000;321.128000000000;322.598000000000;324.072000000000;325.552000000000;327.012000000000;328.510000000000;329.995000000000;331.482000000000;332.956000000000;334.448000000000;335.957000000000;337.470000000000;338.983000000000;340.500000000000;342.010000000000;343.525000000000;345.026000000000;346.546000000000;348.079000000000;349.624000000000;]; %K

while(done==0 && str2num(get(handles.editSTOP,'String'))==0 && count<30)
    count=count+1;
    
    %% Solve DE
    OPTIONS=odeset('RelTol',1e-6,'AbsTol',1e-6);
    i=1:Nx;
    T_ini(i,1)=Tf(i,1);
    i=1:Nx;
    T_ini(Nx+i,1)=Tr(i,1);
    
    TSPAN=[0:tau/Nt:tau];
    
    [time,T]=ode45(@(time,T)dTdt_function(time,T,xr,Ac,as,eps,tau,Nx,L,TH,TC,massflowfun2,muoHfun2,fluidfun2,mcmpropfun2,bedcorrfun,pacCOOKED,dh,TCurie,mdot_amp,massflow_inp,muoH_max,maginp),TSPAN,T_ini,OPTIONS);
    if str2num(get(handles.editSTOP,'String'))==1;
        break;
    end
    %% Extract the fluid and solid temperatures
    [M,g]=size(T);
    for j=1:M
        for i=1:Nx
            Tf(i,j)=T(j,i);
            Tr(i,j)=T(j,Nx+i);
        end
    end
    
    T_red = Tf/T_ref;
    for i=1:Nx;
        Tr_new(i,:)=Tr(i,:)./a(i);
    end
    
    if str2num(get(handles.editSTOP,'String'))==1;
        break;
    end
    
    for j=1:Nt+1
        hr(:,j)=interp1(T_0,h_0,Tr_new(:,j),'pchip');
        hf(:,j)=(polyval(int_cp,T_red(:,j))-polyval(int_cp,1))*T_ref; %specific enthalpy of fluid (h_ref=0 J/kg)
    end
    
    if str2num(get(handles.editSTOP,'String'))==1;
        break;
    end
    
    Ef=eps*rhof*hf*Ac*L/Nx; %fluid energy entrained in one bed of the regenerator
    Er=(1-eps)*rhor(1)*hr*Ac*L/Nx; %regenerator energy for one bed
    
    dE_cycle=sum(abs(Er(:,Nt+1)-Er(:,1)))+sum(abs(Ef(:,Nt+1)-Ef(:,1)));%absolute value of the change in energy over one cycle
    Emax=max(sum(Ef,1)+sum(Er,1)); %maximum energy in the solid and fluid during previous cycle
    Emin=min(sum(Ef,1)+sum(Er,1)); %minimum energy in the solid and fluid during previous cycle
    err(count)=abs(dE_cycle/(Emax-Emin)); %figure of merit for cyclical steady state convergence
    
    set(handles.Countoutput,'String',num2str(count));
    set(handles.erroutput,'String',num2str(err(count)));
    drawnow();
    
    disp(['Iteration#: ' num2str(count)])
    disp(['Error: ' num2str(err(count))])
    fprintf('\n');
    
    if err(count)<tol %system has reached steady state, exit while loop
        done=1;
    else
        Tr(:,1)=Tr(:,Nt+1); %initial regenerator temp for next cycle
        Tf(:,1)=Tf(:,Nt+1);  %initial fluid temp for next cycle
    end
    
    if err(count)<=err_s
        err_s=err(count);
        Tf_s=Tf;
        Tr_s=Tr;
        dE_cycle_s=dE_cycle;
    end
    
    % Plots
    axes(handles.FluidTemperature);
    plot(t(1:Nt),Tf(1,1:Nt),'k')
    hold on
    plot(t(1:Nt),Tf(Nx/2,1:Nt),'b')
    plot(t(1:Nt),Tf(Nx,1:Nt),'r')
    h=gca;
    axes(h)
    xlabel('Time (s)')
    ylabel('Fluid Temperature (K)')
    legend('hot end','middle','cold end')%,'Location','NorthEastOutside')
    grid on
    hold off
    %title('Fluid temperature as a function of time at the cold side, hot side and middle of the regenerator bed');
    
    axes(handles.SolidTemperature);
    plot(t(1:Nt),Tr(1,1:Nt),'k')
    hold on
    plot(t(1:Nt),Tr(Nx/2,1:Nt),'b')
    plot(t(1:Nt),Tr(Nx,1:Nt),'r')
    h=gca;
    axes(h)
    xlabel('Time (s)')
    ylabel('Solid Temperature (K)')
    legend('hot end','middle','cold end')%,'Location','NorthEastOutside')
    grid on
    hold off
    %title('Magnetocaloric material temperature as a function of time at the cold side, hot side and middle of the regenerator bed');
    
    axes(handles.SolidMesh);
    mesh(t(1:Nt),xr,Tr(:,1:Nt))
    h=gca;
    axes(h)
    xlabel('Time (s)')
    ylabel('Axial Location (m)')
    zlabel('Solid Temperature (K)')
    %title('Magnetocaloric material temperature distribution as a function of time and position');
    view(-112,45);
    drawnow();
    
end
%%
Tf=Tf_s;
Tr=Tr_s;
dE_cycle=dE_cycle_s;
%%
for j=1:Nt
    [muf(:,j),kf(:,j),cf(:,j),rhof,hf(:,j),sf(:,j)]=fluidfun(Tf(:,j));
    Ref(:,j)=dh*abs(v(j))*rhof./(muf(:,j)); %Reynolds number based on hydraulic diameter
    Prf(:,j)=cf(:,j).*muf(:,j)./(kf(:,j)); %Prandtl number
    [cmuoH(:,j),dsdmuoH(:,j),rhor,kr(:,j),sr(:,j),hr(:,j),vM_irr(:,j)]=mcmpropfun(Tr(:,j),TCurie,muoH(:,j));
    [Nuf(:,j),ff(:,j),kstat(:,j),kdisp(:,j)]=bedcorrfun(pacCOOKED,Ref(:,j),Prf(:,j),kf(:,j), kr(:,j),v(j),muf(:,j),rhof,cf(:,j),tau);
    dP(:,j)=ff(:,j).*(v(j)).^2/(2*dh)*rhof*L/Nx; %Pressure drop in Pa, based on hydraulic diameter, this is correct
end

%calculate pressure drop in the regenerator and pump work
for j=1:Nt
    dWpump(:,j)=abs(mdot(j)/rhof*dP(:,j)); %W
    deltaP(j)=sum(dP(:,j));
end
Wpump=sum(sum(dWpump,1),2)*dt; %pump work in J

dwell=dr;
n_beds=nbed;

TCedg=0.5*(1.5*Tf(Nx,1:Nt)-0.5*Tf(Nx-1,1:Nt))+0.5*(1.5*Tf(Nx,2:Nt+1)-0.5*Tf(Nx-1,2:Nt+1)); %Fluid temperature at the hot edge of the bed.  This temp is extrapolated from the last two nodes in the bed
[mufCedg,kfCedg,cfCedg,rhof,hfCedg,sfCedg]=fluidfun(TCedg);
Tout_avg=trapz(t(1:Nt/2),TCedg(1:(Nt)/2)')/(t((Nt)/2)-t(1));%*pi/(2*mdot_amp)
eff=(TH-Tout_avg)/(TH-TC);
NTU=Nuf(1,1)*kf(1,1)*Ac*L*as/(dh*mdot_amp*cf(1,1));

U=mdot_amp*dwell*cf(1,1)*tau/(2*Ac*L*(1-eps)*rhor(1)*cmuoH(1,1));
Crat=Ac*L*(1-eps)*rhor(1)*cmuoH(1,1)/(mdot_amp*cf(1,1)*tau/2);
t_conv=toc; %this determines how long the model took to converge

%%%Calculate dimensionless parameters
V=Ac*L;
cr_avg=mean(mean(cmuoH));
cf_avg=mean(mean(cf));
Cr=mean(rhor)*V*(1-eps)*cr_avg;
Cf=rhof*V*(eps)*cf_avg;
R=Cf/Cr;
lambda=tau/2*(1-dwell);
U=mdot_amp*cf_avg*lambda/(Cr+Cf);
mdot_max=max(mdot);
z=0;

for i=1:size(mdot)
    if mdot(i)==mdot_max
        z=z+1;
        Nu_flow(z)=Nuf(1,i);
    end
end
Nu_avg=mean(Nu_flow);
NTU=Nu_avg*kf(1,19)*Ac*L*as/(dh*mdot_amp*cf_avg);

%% Calculate net enthalpy and entropy fluxes

for i=1:Nx
    hflux(i,1)=sum(mdot.*hf(i,1:Nt)')*tau/Nx;
    sflux(i,1)=sum(mdot.*sf(i,1:Nt)')*tau/Nx;
end

THedg=0.5*(1.5*Tf(1,1:Nt)-0.5*Tf(2,1:Nt))+0.5*(1.5*Tf(1,2:Nt+1)-0.5*Tf(2,2:Nt+1)); %Fluid temperature at the hot edge of the bed.  This temp is extrapolated from the last two nodes in the bed
[mufHedg,kfHedg,cfHedg,rhof,hfHedg,sfHedg]=fluidfun(THedg);
hflux(Nx+1,1)=sum(mdot.*hfCedg')*tau/Nt;

%Calculate system performance
ref_load=0; heat_load=0; Q_max1=0; Q_max2=0;QH=0;
for i=1:Nt
    if mdot(i)>0
        ref_load=ref_load-mdot(i)*(hfCedg(i)-hfC)*dt; %refrigeration in J supplied to the cold reservoir for one cycle by one bed
    end
    if mdot(i)<0
        heat_load=heat_load-mdot(i)*(hfHedg(i)-hfH)*dt; %heat transferred from the hot reservoir in J to the surroundings for one cycle by one bed
        QH=QH-mdot(i)*(hfCedg(i)-hfH)*dt;
        Q_max2=Q_max2+mdot(i)*(hfH-hfC)*dt;
        epsilon=QH/Q_max2;
    end
end

ref_cap=ref_load/tau*n_beds; %refrigeration rate in W supplied to the cold reservoir by the entire regenerator
heat_rej=heat_load*n_beds/tau; %heat transfer rate from the hot reservoir in W to the surroundings by the entire regenerator
Wmag=+heat_load-ref_load-Wpump;%-dE_cycle; %magnetic work in J
Wmotor=Wmag/n_motor; % Work of the motor in J done by the electric motor for each bed
Qpump=Wpump/(n_pump*tau)*n_beds; %rate of work applied to the pump in W for all beds in the regenerator
Wdotmag=Wmag*n_beds/tau;    %rate of magnetic work in W
Wdotpump=Qpump;
COP=ref_load/(Wmotor+Wpump/n_pump); %coefficient of performance
COP_Carnot=(TC)/(TH-TC); %Carnot efficiency for a cycle operating between the specified hot and cold reservoir temperatures