%% This function calculates the Nusselt number, friction factor, static
%% axial conduction that occurs simultaneously through the solid and fluid
%% regenerator matrix, and axial conduction due to axial dispersion in the
%% fluid for a regenerator constructed of equal diameter smooth spheres.
%% Many inputs for this function are contained in the array 'inputs'.  The
%% values from the array are assigned to individual variables and the
%% geometry-specific dimensionless numbers are calculated.
%% The inputs array can contain a variable number of inputs and therefore
%% can describe many different regenerator geometries.  The processed
%% variables such as hydraulic diameter and porosity in the x direction are
%% generated by pack_geom.m.  See that function for more detail.  The first
%% four entries in the 'inputs' array are the unprocessed regenerator data
%% and they are not used by this function.  For SphPart2.m the array 
%% 'inputs' has the following variables stored in the indices of the array:

%%  1       dummy variable, not used
%%  2       dummy variable, not used
%%  3       dummy variable, not used
%%  4       dummy variable, not used
%%  5       dummy variable, not used
%%  6       eps, volumetric or overall porosity 0 < eps < 1
%%  7       dh, hydraulic diameter in meters
%%  8       cNu, Nusselt number scaling factor [dimensionless]
%%  9       cff, friction factor scaling factor [dimensionless]
%%  10     cnk, axial conduction and dispersion scaling factor
%%           [dimensionless].  This scaling factor applies to both the
%%           static conduction and axial dispersion.

function[Nu_corr,f,kstatic,kdisp]=SphPart2(inputs,Re,Pr,kf,kr,v,muf,rhof,cf,tau);

%% parse the inputs array to determine geometry-specific inputs

eps=inputs(6); %volumetric porosity
dh=inputs(7); %hydraulic diameter in meters
cNu=inputs(8); %Nusselt number scaling factor
cff=inputs(9); %friction factor scaling factor
cnk=inputs(10); %axial conduction and dispersion scaling factor

[Nx Nt]=size(Re); %determine the number of time steps and spatial steps in the Reynolds number matrix

Re1=3*(1-eps)/2*Re; %Reynolds number based on particle diameter
dp=3/2*(1-eps)/eps*dh;

%% Calculate friction factor from Kaviany (1995)

A=180; %for Ergun model from Fand et al
B=1.8; %for Ergun model from Fand et al
% for j=1:Nt
%     f(:,j)=cff*(A./(4.5*Re(:,j))+B/3); %this is incorrect
% end
for j=1:Nt
    f(:,j)=cff*(A*8./(9*eps*Re(:,j))+4*B/(3*eps^2));  %this is correct
end

%%convert Nusselt correlation for Re1 and Nu1 to Re and Nu then correct the
%%Nusselt number correlation for internal temperature gradients

Nu1=cNu*(2+1.1*Re1.^(0.6).*Pr.^(1/3)); %from Wakao 1982, based on particle diameter

% Calculate Biot number and the heat transfer degradation factor
Bi=(Nu1.*kf)./(2*kr);
%Bi=0;
Fo=kf*tau./(rhof*cf*(dp/2)^2);
chi=Fo.*exp(0.246196-0.84878*log(Fo)-0.05639*(log(Fo)).^2); %from J of Heat Transfer paper
Nu1=Nu1./(1+chi.*Bi/5); %corrected heat transfer coefficient 
Nu_corr=2/3*eps/(1-eps)*Nu1; %make definition of Nu consistent with hydraulic diameter def

%%Calculate static conductivity according to Hadley (1986) and axial
%%dispersion from Kaviany (1995)

if eps<0.0827
    a0=10^(-4.898*eps);
elseif eps<0.298
    a0=10^(-.405-3.154*(eps-0.0827));
elseif eps<0.58
    a0=10^(-1.084-6.778*(eps-0.298));
end
f0=0.8+0.1*eps;
kstatic=cnk*kf.*((1-a0)*(eps*f0+kr./kf*(1-eps*f0))./(1-eps*(1-f0)+kr./kf*eps*(1-f0))+a0*(2*(kr./kf).^2*(1-eps)+(1+2*eps)*kr./kf)./((2+eps)*kr./kf+1-eps)); %W/m2-K, Hadley (1986) from Kaviany
kdisp=cnk*.75*eps*kf.*Pr.*Re1; %W/m2-K, effective conduction in fluid due to axial dispersion