%% This function generates a mass flow profile with two dwell periods and an
%% s-shaped transition to the maximum absolute mass flow rate.  This flow
%% can be unbalanced according to the unbalance input, which ranges from 0
%% to 1.  The mass flow in the positive direction begins at time = delay.
%%
%% The ramp in flow is smoothed by using a cubic function to go from 0 
%% flow to a blow period.  This makes derivative of the mass flow rate 
%% always continuous and makes the numerical model more stable.  Because 
%% the ramped portions of the mass blow period reduce the total flow 
%% through the bed during the blow period, the amplitude of the mass flow 
%% rate is corrected to give the same total mass flow as an on/off flow.  
%% The flow rate is never set to 0 because the implicit model
%% cannot handle a 0 mass flow rate and the same function is used by the
%% implicit and explicit models.
%%
%% The inputs to the function are as follows:
%%      time        time or temporal grid of one cycle of the system
%%      tau         time for one complete cycle (s)
%%      mdot_amp    maximum amplitude of the mass flow rate (kg/s)
%%      inputs      array that contains function specific flow
%%                  characteristics.  The array containts the folling
%%                  variables:
%%       dwell       dwell ratio in the range 0-1
%%       delay       delay time, in seconds, before first blow period
%%                   begins
%%       ramp        fraction of each blow period that is spent ramping up
%%                   or ramping down.  0 < ramp < 0.5
%%       unbal       unbalance fraction.  Fraction of flow in the negative
%%                   direction to flow in positive direction.  
%%                   -1 < unbal < 1 

function[mdot]=mdotsmoothunbal_delay2(time,tau,mdot_amp,inputs);

%% parse inputs array

dwell=inputs(1); %[unitless]
delay=inputs(2); %s
ramp=inputs(3);
unbal=inputs(4)/2; %offset the flow in each direction by 0.5*unbalance

t1=delay;
t2=delay+tau/2*(1-dwell)*ramp;
t3=delay+tau/2*(1-dwell)-tau/2*(1-dwell)*ramp;
t4=delay+tau/2*(1-dwell);
t5=tau/2;
%[n,g]=size(time);
%n=n-1;
if delay>(tau/2*dwell)
    fprintf('delay is too long\n')
end
%begin=0;
%fin=0;
mdot0=mdot_amp;
mdot_amp=mdot_amp*(1-unbal);
%for j=0:n
    if(time<t1)
        mdot=.001*mdot_amp;
    elseif(time<t2)
        mdot=.001*mdot_amp-2*mdot_amp/(t2-t1)^3*(time-t1)^3+3*mdot_amp/(t2-t1)^2*(time-t1)^2;
    elseif(time<t3)
        mdot=mdot_amp;
    elseif(time<t4)
        mdot=.001*mdot_amp+2*mdot_amp/(t4-t3)^3*(time-t3)^3-3*mdot_amp/(t4-t3)^2*(time-t3)^2+mdot_amp;
    elseif(time<=t5)
        mdot=.001*mdot_amp;
    else
        time=time-t5;
        mdot_amp=-mdot0*(1+unbal);
        if(time<t1)
            mdot=.001*mdot_amp;
        elseif(time<t2)
            mdot=.001*mdot_amp-2*mdot_amp/(t2-t1)^3*(time-t1)^3+3*mdot_amp/(t2-t1)^2*(time-t1)^2;
        elseif(time<t3)
            mdot=mdot_amp;
        elseif(time<t4)
            mdot=.001*mdot_amp+2*mdot_amp/(t4-t3)^3*(time-t3)^3-3*mdot_amp/(t4-t3)^2*(time-t3)^2+mdot_amp;
        elseif(time<=t5)
            mdot=.001*mdot_amp;
        else
            mdot=0.001*mdot_amp;
        end
    end
%end
mdot=-1.33111438649762*mdot;
%% correct the amplitude of the mass flow rate to achieve the correct
%% average mass flow rate.

% marea=trapz(time(begin:fin),mdot(begin:fin));
% square=(time(fin)-time(begin))*mdot0*(1-unbal);
% mdot=-square/marea*mdot;